#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <unistd.h>
#include <openobex/obex.h>

#define HEADER_CREATOR_ID 0xcf
#define HEADER_DESCRIPTION 0x05

obex_t *handle = NULL;
volatile int finished = 0;

void empeg_command(char *serial_cmd, int notify_fd) {
	//fprintf(stderr, "Empire: Performing Serial command: %s", serial_cmd);
	write(notify_fd, serial_cmd, strlen(serial_cmd));
}

void fid_command(const uint8_t *fid, int notify_fd) {
	char *serial_cmd;
	serial_cmd = malloc(strlen(fid) + 9);
	snprintf(serial_cmd, strlen(fid) + 9, "SERIAL=%s\n", fid);
	empeg_command(serial_cmd, notify_fd);
	free(serial_cmd);
}

void put_done(obex_object_t *object) {
	obex_headerdata_t header_data;
	uint8_t header_ident;
	int i, count, hlen, body_len = 0;
	int notify_fd;	// this is /proc/empeg_notify
	const uint8_t *uint8_t_body = NULL;
	char *name = NULL, *namebuf = NULL;
	char fid[14], serial_cmd[39];
	char action[12];

	while(OBEX_ObjectGetNextHeader(handle, object, &header_ident, &header_data, &hlen)) {
		switch(header_ident) {

		case OBEX_HDR_BODY:
			uint8_t_body = header_data.bs;
			body_len = hlen;
			break;

		case OBEX_HDR_NAME:
			if((namebuf = malloc(hlen / 2))) {
				OBEX_UnicodeToChar(namebuf, header_data.bs, hlen);
				name = namebuf;
			}
			break;

		case HEADER_DESCRIPTION:	//recieved a part of the description
		case OBEX_HDR_LENGTH:		//recieved the length of the header
		case HEADER_CREATOR_ID:		//recieved the creator id
		default:			//other header we don't care about
		}
	}

	if(strncmp(name, "palantir_", 9) == 0) {
		if((notify_fd = open("/proc/empeg_notify", O_WRONLY)) != -1) {
			if(strcmp(name, "palantir_insert.empeg") == 0) {
				//insert mode
				for(i = body_len - 8; i >= 0; i -= 8) {
					strcpy(fid, "#XXXXXXXX0!");
					memcpy(fid + 1, (uint8_t_body + i), 8);
					fid_command(fid, notify_fd);
				}
				strcpy(action, "inserted");
			} else if(strcmp(name, "palantir_append.empeg") == 0) {
				//append mode
				for(i = 0; i <= body_len - 8; i += 8) {
					//hijack interprets plus signs
					//convert to url-ecoded plus %2B
					strcpy(fid, "#XXXXXXXX0%2B");
					memcpy(fid + 1, (uint8_t_body + i), 8);
					fid_command(fid, notify_fd);
				}
				strcpy(action, "appended");
			} else if(strcmp(name, "palantir_replace.empeg") == 0) {
				//replace mode
				//play the first fid, then append to the rest
				strcpy(fid, "#XXXXXXXX0 ");
				memcpy(fid + 1, (uint8_t_body), 8);
				fid_command(fid, notify_fd);
				for(i = 8; i <= body_len - 8; i += 8) {
					strcpy(fid, "#XXXXXXXX0%2B");
					memcpy(fid + 1, (uint8_t_body + i), 8);
					fid_command(fid, notify_fd);
				}
				strcpy(action, "in playlist");
			} else if(strcmp(name, "palantir_enqueue.empeg") == 0) {
				//enqueue mode
				for(i = 0; i <= body_len - 8; i += 8) {
					strcpy(fid, "#XXXXXXXX0-");
					memcpy(fid + 1, (uint8_t_body + i), 8);
					fid_command(fid, notify_fd);
				}
				strcpy(action, "enqueued");
			}
	
			count = body_len / 8;
			if(count >= 10) {
				sprintf(serial_cmd, "popup 5 Empire: %d songs %s", count, action);
			} else if(count > 1) {
				sprintf(serial_cmd, "popup 5 Empire: %d songs %s", count, action);
			} else {
				sprintf(serial_cmd, "popup 5 Empire: %d song %s", count, action);
			}
			empeg_command(serial_cmd, notify_fd);
			
			//close the file handle
			close(notify_fd);
		}
	}
	free(namebuf);
}

void server_request(obex_object_t *object, int event, int cmd) {
	switch(cmd) {
	case OBEX_CMD_SETPATH:
		OBEX_ObjectSetRsp(object, OBEX_RSP_CONTINUE, OBEX_RSP_SUCCESS);
		break;
	case OBEX_CMD_PUT:
		OBEX_ObjectSetRsp(object, OBEX_RSP_CONTINUE, OBEX_RSP_SUCCESS);
		put_done(object);
		break;
	case OBEX_CMD_CONNECT:
	case OBEX_CMD_DISCONNECT:
		OBEX_ObjectSetRsp(object, OBEX_RSP_SUCCESS, OBEX_RSP_SUCCESS);
		break;
	default:
		OBEX_ObjectSetRsp(object, OBEX_RSP_NOT_IMPLEMENTED, OBEX_RSP_NOT_IMPLEMENTED);
		break;
	}
	return;
}

void obex_event_handler(obex_t *handle, obex_object_t *object, int mode, int event, int obex_cmd, int obex_rsp) {
	switch (event)	{
	case OBEX_EV_PROGRESS:	// a piece has been recieved
		break;
	case OBEX_EV_REQDONE:
		if(obex_cmd == OBEX_CMD_DISCONNECT) {
			finished = 1;
			OBEX_TransportDisconnect(handle);
		}
		break;
	case OBEX_EV_REQHINT:
		switch(obex_cmd) {
		case OBEX_CMD_PUT:		//Getting object
		case OBEX_CMD_CONNECT:
		case OBEX_CMD_DISCONNECT:
			OBEX_ObjectSetRsp(object, OBEX_RSP_CONTINUE, OBEX_RSP_SUCCESS);
			break;
		default:
			OBEX_ObjectSetRsp(object, OBEX_RSP_NOT_IMPLEMENTED, OBEX_RSP_NOT_IMPLEMENTED);
			break;
		}
		break;
	case OBEX_EV_REQ:
		// Comes when a server-request has been completed
		server_request(object, event, obex_cmd);
		break;
	case OBEX_EV_LINKERR:
		finished = 1;
		break;
	default:
	}
}

int main(int argc, char **argv) {
	char linkname[128];
	char pathname[128];
	char irattachCommand[128];
	int ilen = -1, seconds = 10;  //default to 10 second wait
	
	//wait the number of seconds passed on the command line, if any
	if(argc == 2) seconds = atoi(argv[1]);
	
	fprintf(stderr, "Empire: Version 0.30 starting...\n");
	
	//background
	daemon(0, 0);
	
	//get the "working directory"
	snprintf(linkname, sizeof(linkname), "/proc/%d/exe", getpid());
	ilen = readlink(linkname, pathname, 128);
	pathname[ilen - 7] = '\0';

	//first we have to start irattach AFTER the player starts
	sleep(seconds);
	snprintf(irattachCommand, sizeof(irattachCommand), "%s/irattach /dev/ttyS2", pathname);
	system(irattachCommand);

	//now begin the IrDA stuff
	handle = OBEX_Init(OBEX_TRANS_IRDA, obex_event_handler, 0);
	while(1) {
		IrOBEX_ServerRegister(handle, "OBEX");
		while (!finished) OBEX_HandleInput(handle, 1);
		finished = 0;
	}

	return 0;
}

